//
//  NSArray+HigherOrderFunctions.h
//  SubjectiveKit
//
//  Created by Mizushima Kota on 12/05/02.
//  Copyright (c) 2012 onion-lang.org. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "FunctionTypes.h"
#import "Monoid.h"

/**
 * A category that provides some convenient higher-order
 * functions to |NSArray|.  Because the functions in this 
 * category has no side effect, users don't need to
 * worry about it.
 */
@interface NSArray (HigherOrderFunctions)

/**
 * For each element with index, call 
 * fn(element, index) sequentially.
 */
- (void)eachWithIndex:(Fn2)fn;

/**
 * Returns a new |NSArray| generated from self.
 * The each element of the |NSArray| is the result of
 * applying |fn(element)|.
 */
- (NSArray*)map:(Fn1)fn;

/**
 * Returns a new |NSArray| by concatenating
 * each NSArray generated from |fn(e)|.
 * In Haskell, this method is known as (>>=).
 */
- (NSArray*)bind:(Fn1)fn;

/*
 *  From the first, take elements until !pred(e), which e is element of self and return the |NSArray|.
 */
- (NSArray*)takeWhile:(Pred1)pred;

/*
 *  From the first, drop elements until !pred(e), which e is element of self and rest of the |NSArray|.
 */
- (NSArray*)dropWhile:(Pred1)pred;

/**
 * Returns a new |NSArray| excluding the element
 * which doesn't satisfy |pred(e)|.
 */
- (NSArray*)filter:(Pred1)pred;

/**
 * Returns a new |NSArray| excluding the element
 * which satisfy |pred(e)|.  This method is inverse
 * of |filter|.
 */
- (NSArray*)filterNot:(Pred1)pred;

/**
 * Returns an object.  The result object is
 * |fun(... fun(fun(seed, e1), e2), ...en)|, which
 * |n| is |self.count|.
 */
- (id)foldLeft:(id)seed of:(Fn2)fun;

/**
 * Returns an object.  The result object is
 * |fun(en, ..., fun(e2, fun(e1, seed)))|, which
 * |n| is |self.count|.
 */
- (id)foldRight:(id)seed of:(Fn2)fun;

/**
 * Returns a |NSString| by concatenating each element
 * of |self| separated by |separator|.
 */
- (NSString*)join:(NSString*)separator;

/**
 * Returns new |NSSet| from this array.
 */
- (NSSet*)toSet;

/**
 * Sum of all element using |concept|, which is a |NSObject<Monoid>*|.
 * element type should be equal to concept type
 */
- (NSObject*)sum:(NSObject<Monoid>*)concept;


@end
